import logging
import os
import re
import sys
import traceback
from requests.exceptions import HTTPError, ConnectionError

from . import util_gerrit, util_logging, cli


def clone_repositories(repositories, name_filter, dry_run):
    dirs = [d for d in os.listdir(".") if os.path.isdir(d)]

    count = 0

    for repository in repositories:
        if name_filter is not None \
                and re.match(name_filter, repository["name"]) is None:
            logging.debug("skipping filtered repository \"%s\"...",
                          repository["name"])
        elif repository["name"] in dirs:
            logging.info("skipping existing repository \"%s\"...",
                         repository["name"])
        else:
            logging.info("cloning repository \"%s\"...\n"
                         "clone command: %s\n",
                         repository["name"], repository["clone_command"])
            count += 1

            if not dry_run:
                os.system(repository["clone_command"])

    logging.info("finished cloning (%d) repositories.", count)


def run(args):
    logger = util_logging.adjust_logger()

    args = cli.parse_args(args)
    if not args:
        return

    logger.setLevel(util_logging.get_log_level(args.log_level))
    args = cli.prompt_for_missing_args(args)

    repositories = util_gerrit.get_repositories(
        args.gerrit_user, args.gerrit_password, args.gerrit_url)

    clone_repositories(repositories, args.filter, args.dry_run)


def main():
    args = sys.argv[1:]
    try:
        run(args)
    except KeyboardInterrupt:
        logging.error("Terminated on user input.")
        exit()
    except (HTTPError, ConnectionError):
        logging.error("Error connecting to server."
                      " Use '--log-level debug' to see details.")
        logging.debug(traceback.format_exc())
    except Exception:
        logging.error("An unknown error have occurred."
                      " Use '--log-level debug' to see details.")
        logging.debug(traceback.format_exc())
    sys.stdout.flush()


if __name__ == "__main__":
    main()



import os
from setuptools import setup, find_packages

this_directory = os.path.abspath(os.path.dirname(__file__))


def read_file(path):
    with open(os.path.join(this_directory, path), encoding='utf-8') as f:
        return f.read()


long_description = read_file("README.md")


setup(
    name="gerrit-sync",
    version=0.2,
    description="a command line tool to easily"
                "clone gerrit repositories in bulk.",
    description_content_type='text/markdown',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords="repository gerrit git clone sync",
    author="Fırat Civaner",
    author_email="fcivaner@gmail.com",
    license="MIT License",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": ["gerrit-sync=gerrit_sync.__main__:main"],
    },
    install_requires=[
        "pygerrit2",
        "requests"
    ]
)



import argparse
import logging
import os
from getpass import getpass


def get_parser():
    parser = argparse.ArgumentParser(description="gerrit-sync")

    parser.add_argument("--gerrit-user",
                        help="Username for access to gerrit API."
                             " If not provided via this argument or the"
                             " environment variable GERRIT_USER,"
                             " it will be asked via the command prompt.",
                        default=os.environ.get("GERRIT_USER"))

    parser.add_argument("--gerrit-password",
                        help="** Password for access to gerrit API."
                             " If not provided via this argument or the"
                             " environment variable GERRIT_PASSWORD,"
                             " it will be asked via the command prompt."
                             " ** WARNING **: Providing the password via the"
                             " command line argument is not recommended "
                             " due to the risk of your password being"
                             " recorded in your shell history.",
                        default=os.environ.get("GERRIT_PASSWORD"))

    parser.add_argument("--gerrit-url",
                        help="Base URL of the gerrit API."
                             " If not provided via this argument or the"
                             " environment variable GERRIT_URL,"
                             " it will be asked via the command prompt."
                             " example: https://gerrit.mydomain.com",
                        default=os.environ.get("GERRIT_URL"))

    parser.add_argument("--filter",
                        help="A regular expression to filter project names."
                             " If not provided, projects are not filtered.")

    parser.add_argument("--dry-run",
                        help="If specified, just display the operations to be"
                             " done but do not perform them.",
                        action="store_true")

    parser.add_argument("--log-level",
                        choices=["error", "info", "debug"],
                        default="info")

    return parser


def parse_args(args):
    argument_parser = get_parser()
    args = argument_parser.parse_args(args)
    valid = validate_args(args)
    if valid:
        return args
    else:
        argument_parser.print_help()
        return None


def validate_args(args):
    if args.gerrit_url is None:
        logging.error("gerrit_url is not provided.")
        return False

    if args.gerrit_user is None:
        logging.error("gerrit_user is not provided.")
        return False

    return True


def prompt_for_missing_args(args):
    if args.gerrit_password is None:
        args.gerrit_password = getpass("password: ")
    return args



import logging

from pygerrit2 import GerritRestAPI, HTTPBasicAuth


def get_repositories(username, password, gerrit_url):
    auth = HTTPBasicAuth(username, password)
    rest = GerritRestAPI(url=gerrit_url, auth=auth)

    # get server information to derive clone commands
    logging.debug("getting server information..")
    server_info = rest.get("/config/server/info")
    clone_command = server_info["download"]["schemes"]["ssh"]["clone_commands"]
    clone_command = clone_command["Clone with commit-msg hook"]

    # get all projects
    logging.debug("getting project information..")
    repositories = rest.get("/projects/")

    result = []
    for repository_name in repositories.keys():
        project_clone_command = clone_command \
            .replace("${project}", repository_name) \
            .replace("${project-base-name}", repository_name)

        result.append({
            "name": repository_name,
            "scm": "git",
            "source": "gerrit",
            "source_username": username,
            "clone_command": project_clone_command
        })
    return result
